//-----------------------------------------------------------------
//
// Title       : in_shift
// Design      : interface
// Author      : Sindre
// File        : in_shift.v
//
//-----------------------------------------------------------------
//
// Description : input interface 8-bit input 5-bit output
//				 in all, 128 bits are shifted in using a 8-bit bus
//				 and the symbols are shifted out 5-bit at the time
//
//-----------------------------------------------------------------
`timescale 1 ns / 1 ps

//{module {in_shift}}
module in_shift(
	input clk,
	input reset,
	input enable,
	input [7:0] in,
	input i_start,
	output reg s_ready,
	output reg [4:0] out
	);
	
	//constant declaration
	parameter size = 4;
	parameter IDLE = 4'b0000;
	parameter STEP1 = 4'b0001;
	parameter STEP2 = 4'b0011;
	parameter STEP3 = 4'b0010;
	parameter STEP4 = 4'b0110;
	parameter STEP5	= 4'b0111;
	parameter STEP6 = 4'b0101;
	parameter STEP7 = 4'b0100;
	
	//internal variables
	reg [size-1:0] state;
	reg [size-1:0] next_state;
	reg [4:0] S [0:26]; 	//internal storage for symbol values
	reg [5:0] i;			//internal counter
	reg [3:0] l;			//internal counter
		
	
	//--------------FSM-------------------//
	always @(posedge clk or negedge reset)
	begin
		if(reset == 1'b0)begin
			state <= IDLE;
		end else if(enable) begin
			state <= next_state;
		end
	end
	
	//The comments shows which symbols that are being shifted
	//in and out each cycle
	always @(state or i or i_start)
	begin
		case(state)
			IDLE: begin
				if(i_start) begin//i=0
					next_state = STEP1;
				end else begin
					next_state = IDLE;
				end	
			end
				
			STEP1: begin //i=1, 9, 17
				if(i == 1)begin
					next_state = STEP2;
				end else begin
					next_state = STEP2;
				end
			end
				
			STEP2: begin//i=2, 10, 18
				next_state = STEP3;
			end
			
			STEP3: begin//i=3, 11, 19
				next_state = STEP4;
			end
				
			STEP4: begin//i=4, 12, 20
				if(i == 20)begin
					next_state = STEP6;
				end else begin
					next_state = STEP5;
				end
			end
				
			STEP5: begin //i=8, 16, 
				next_state = STEP1;
			end
			
			STEP6: begin//i=24
				next_state = STEP7;
			end
			
			STEP7: begin
				if(i == 26)begin
					next_state = IDLE;
				end else begin
				next_state = STEP7;
				end
			end

			default: next_state = IDLE;
		endcase
	end
	
	always @(posedge clk or negedge reset)
	begin
		if(reset == 1'b0)begin
			s_ready <= 0;
			out[4:0] <= 0;
			S[0] <= 0;		S[11] <= 0;		S[21] <= 0;
			S[1] <= 0;		S[12] <= 0;		S[22] <= 0;
			S[2] <= 0;		S[13] <= 0;		S[23] <= 0;	
			S[3] <= 0;		S[14] <= 0;		S[24] <= 0;
			S[4] <= 0;		S[14] <= 0;		S[25] <= 0;
			S[5] <= 0;		S[15] <= 0;		S[26] <= 0;
			S[6] <= 0;		S[16] <= 0;	
			S[7] <= 0;		S[17] <= 0;		
			S[8] <= 0;		S[18] <= 0;	
			S[9] <= 0;		S[19] <= 0;
			S[10] <= 0;		S[20] <= 0;
			i <= 0;
			l <= 0;
		end else if(enable) begin
			case(state)
				IDLE: begin
					if(i_start) begin//i=0
						s_ready <= 1;
						S[i][4:0] <= in[7:3];
						S[i+1][4:2] <= in[2:0];
						i <= i+1;
						out[4:0] <= in[7:3];//0 
					end else begin
						s_ready <= 0;
						out[4:0] <= 0;
					end
				end
				
				STEP1: begin //i=1, 9, 17
					s_ready <= 0;
					S[i][1:0] <= in[7:6];
					S[i+1][4:0] <= in[5:1];
					S[i+2][4] <= in[0];
					
					if(i == 1)begin
						out[4:2] <= S[i][4:2]; //1, 6, 11	   
						out[1:0] <= in[7:6];
						i <= i+1;
					end else begin
						out[4:0] <= S[i-l][4:0];
						i <= i+1;
					end
				end
				
				STEP2: begin//i=2, 10, 18
					S[i+1][3:0] <= in[7:4];
					S[i+2][4:1] <= in[3:0];
					i <= i+1;
					out[4:0] <= S[i-l][4:0];//2, 7 , 12
				end
				
				STEP3: begin//i=3, 11, 19
					S[i+1][0] <= in[7];
					S[i+2][4:0] <= in[6:2];
					S[i+3][4:3] <= in[1:0];
					i <= i+1;
					out[4:0] <= S[i-l][4:0];//3, 8, 13
				end
				
				STEP4: begin//i=4, 12, 20
					S[i+2][2:0] <= in[7:5];
					S[i+3][4:0] <= in[4:0];
					if(i == 20)begin
						i <= i+4;
						l <= l+3;
					end else begin
						i <= i+4;
						l <= l+3;
					end
					out[4:0] <= S[i-l][4:0]; //4, 9, 14
				end
				
				STEP5: begin //i=8, 16, 
					S[i][4:0] <= in[7:3];
					S[i+1][4:2] <= in[2:0];
					i <= i+1;
					out[4:0] <= S[i-l][4:0]; //5, 10,
				end
				
				STEP6: begin//i=24
					S[i][4:0] <= in[7:3];
					S[i+1][4:2] <= in[2:0];
					S[i+1][1:0] <= 2'b00;
					S[i+2][4:0] <= 5'b00000; 
					out[4:0] <= S[i-l][4:0]; // 15
					i <= 16;
					l <= 0;
				end
				
				STEP7: begin
					if(i == 26)begin
						out[4:0] <= S[i][4:0]; //26
						i <= 0;
					end else begin
						// 16, 17, 18, 19, 20, 21, 22, 23, 24, 25
						out[4:0] <= S[i][4:0]; 
						i <= i+1;
					end
				end

				default: begin
					s_ready <= 0;
					out[4:0] <= 0;
					S[0] <= 0;		S[11] <= 0;		S[21] <= 0;
					S[1] <= 0;		S[12] <= 0;		S[22] <= 0;
					S[2] <= 0;		S[13] <= 0;		S[23] <= 0;	
					S[3] <= 0;		S[14] <= 0;		S[24] <= 0;
					S[4] <= 0;		S[14] <= 0;		S[25] <= 0;
					S[5] <= 0;		S[15] <= 0;		S[26] <= 0;
					S[6] <= 0;		S[16] <= 0;	
					S[7] <= 0;		S[17] <= 0;		
					S[8] <= 0;		S[18] <= 0;	
					S[9] <= 0;		S[19] <= 0;
					S[10] <= 0;		S[20] <= 0;
					i <= 0;
					l <= 0;
				end
			endcase
		end
	end
endmodule
