//-----------------------------------------------------------------------------
//
// Title       : inverse_GF32
// Design      : decoder_5bit
// Author      : Sindre
// File        : inverse_GF32.v
//
//-----------------------------------------------------------------------------
//
// Description : Inverse values stored in ROM: 
//				 address_read indicates which decimal value to get inverse
//				 of. data_out then gives inverse of requested value
//
//-----------------------------------------------------------------------------
`timescale 1 ns / 1 ps

//{module {inverse_GF32}}
module inverse_GF32 #(parameter width=5)(clk, re, address_read, data_out);
	input clk;
	input re;
	input [width-1:0] address_read;
	output [width-1:0] data_out;
	reg [width-1:0] data_out;
	wire [width-1:0] mem [0:31];
	
	
	assign	mem[0] = 5'b00000; // 0
	assign	mem[1] = 5'b00001; // 1
	assign	mem[2] = 5'b10010; // 18
	assign	mem[3] = 5'b11100; // 28
	assign	mem[4] = 5'b01001; // 9
	assign	mem[5] = 5'b10111; // 23
	assign	mem[6] = 5'b01110; // 14
	assign	mem[7] = 5'b01100; // 12
	assign	mem[8] = 5'b10110; // 22
	assign	mem[9] = 5'b00100; // 4
	assign	mem[10] = 5'b11001; // 25
	assign	mem[11] = 5'b10000; // 16
	assign	mem[12] = 5'b00111; // 7
	assign	mem[13] = 5'b01111; // 15
	assign	mem[14] = 5'b00110; // 6
	assign	mem[15] = 5'b01101; // 13
	assign	mem[16] = 5'b01011; // 11
	assign	mem[17] = 5'b11000; // 24
	assign	mem[18] = 5'b00010; // 2
	assign	mem[19] = 5'b11101; // 29
	assign	mem[20] = 5'b11110; // 30
	assign	mem[21] = 5'b11010; // 26
	assign	mem[22] = 5'b01000; // 8
	assign	mem[23] = 5'b00101; // 5
	assign	mem[24] = 5'b10001; // 17
	assign	mem[25] = 5'b01010; // 10
	assign	mem[26] = 5'b10101; // 21
	assign	mem[27] = 5'b11111; // 31
	assign	mem[28] = 5'b00011; // 3
	assign	mem[29] = 5'b10011; // 19
	assign	mem[30] = 5'b10100; // 20
	assign	mem[31] = 5'b11011; // 27
	
	
	always @(posedge clk)
	begin
		if(re == 1'b1)begin
			data_out <= mem[address_read];
		end	
	end
endmodule
