//-------------------------------------------------------------------
//
// Title       : syndrome_calc_S0
// Design      : decoder_5bit
// Author      : Sindre
// File        : syndrome_calc_S0.v
//
//------------------------------------------------------------------
//
// Description : Syndrome cell 0: calcualtes first syndrome value
//
//------------------------------------------------------------------
`timescale 1 ns / 1 ps

//{module {syndrome_calc_S0}}
module syndrome_calc_S0 #(parameter width=5'b00101)(
	clk, 
	reset, 
	enable, 
	in, 
	ctrl, 
	si
	);
	
	//input signals
	input clk;			   //clock signal
	input reset;		   //reset signal
	input enable;		   //enable signal
	input [width-1:0] in;  //symbol input
	input ctrl;			   //control signal
	
	//output signals
	output [width-1:0] si; //syndrome output
	
	//ports data type
	wire clk;
	wire reset;
	wire enable;
	wire ctrl;
	wire [width-1:0] in;
	wire [width-1:0] si;
	
	//internal wires
	wire [width-1:0] w_1;
	wire [width-1:0] w_2;
	
	//module instantiations
	//D-register
	D_reg #(.width(width)) REG(
	.clk	(clk),
	.reset	(reset),
	.enable	(enable),
	.d_in	(si),
	.d_out	(w_1)
	);
	
	//AND gate
	AND_gate #(.width(width)) AND(
	.a		(w_1),
	.b		(ctrl),
	.y		(w_2)
	);
	
	//finite field adder
	Adder #(.width(width)) ADD(
	.in1	(w_2),
	.in2	(in),
	.sum	(si)
	);

endmodule
