//-----------------------------------------------------------------------------
//
// Title       : Codeword_Store_modified
// Design      : decoder_5bit
// Author      : Sindre
// File        : Codeword_Store_modified.v
//
//-----------------------------------------------------------------------------
//
// Description : Modified codeword storage module. Enables parallel reading
//				 of data from memory if re1 is selected
//
//-----------------------------------------------------------------------------
`timescale 1 ns / 1 ps

//{module {Codeword_Store_modified}}
module Codeword_Store_modified #(parameter width=5)(
	input clk,						  //clock signal
	input enable,					  //enable signal
	input we,						  //write enable
	input re1,						  //read enable (parallel read)
	input re2,						  //read enable 
	input [width-1:0] address_write,  //write address
	input [width-1:0] address_read1,  //read address 1 (no parallel mode uses address_read1)
	input [width-1:0] address_read2,  //read address 2
	input [width-1:0] data_in,		  //data input
	output reg [width-1:0] data_out1, //data output 1 (no parallel mode uses data_out1)
	output reg [width-1:0] data_out2  //data output 2
	);
	
	reg [width-1:0] mem [0:30];
	reg [4:0] i;
	
	initial
	begin
		for(i=0; i<31; i=i+1)
			mem[i] = 0; 
	end
	
	always @(posedge clk)
	begin
		if(we == 1'b1)begin
			mem[address_write] <= data_in;
		end
		
		if(re1 == 1'b1 && !re2)begin
			data_out1 <= mem[address_read1];
			data_out2 <= mem[address_read2];
		end
		
		if(re2 == 1'b1 && !re1)begin
			data_out1 <= mem[address_read1];
		end
	end

endmodule
