//-----------------------------------------------------------------------------
//
// Title       : Syndrome_cell_parallel_S1
// Design      : decoder_5bit
// Author      : Sindre
// File        : Syndrome_cell_parallel_S1.v
//
//-----------------------------------------------------------------------------
//
// Description : Two parallel syndrome cell for calculating S_i (i=1)
//
//-----------------------------------------------------------------------------
`timescale 1 ns / 1 ps

//{module {Syndrome_cell_parallel_S1}}
module Syndrome_cell_parallel_S1 #(parameter width=5)(
	input clk,					//clock signal
	input reset,				//reset signal
	input enable,				//enable signal
	input ctrl,					//control signal
	input [width-1:0] in_A,		//input A for even symbols
	input [width-1:0] in_B,		//input B for odd symbols
	output [width-1:0] out		//output symdrome value
	);
	
	//internal wires
	wire [width-1:0] ADD1_OUT;
	wire [width-1:0] ADD2_OUT;
	wire [width-1:0] D_OUT;
	wire [width-1:0] AND_OUT;
	wire [width-1:0] MULT1_OUT;
	wire [width-1:0] MULT2_OUT;
	
	assign out = D_OUT;
	
	//module instantiations
	Adder #(.width(width)) ADD01(
	.in1	(in_A),
	.in2	(ADD2_OUT),
	.sum	(ADD1_OUT)
	);
	
	D_reg #(.width(width)) REG(
	.clk	(clk),
	.reset	(reset),
	.enable	(enable),
	.d_in	(ADD1_OUT),
	.d_out	(D_OUT)
	);
	
	AND_gate #(.width(width)) AND(
	.a		(D_OUT),
	.b		(ctrl),
	.y		(AND_OUT)
	);
	
	//multiply with alpha^i*2 => 4
	Mult_x4_5bit #(.width(width)) MULT01(
	.m_in	(AND_OUT),
	.m_out	(MULT1_OUT)
	);
	
	//multiply with alpha^i => 2
	Mult_x2_5bit #(.width(width)) MULT02(
	.m_in	(in_B),
	.m_out	(MULT2_OUT)
	);
	
	Adder #(.width(width)) ADD02(
	.in1	(MULT1_OUT),
	.in2	(MULT2_OUT),
	.sum	(ADD2_OUT)
	);

endmodule
