//------------------------------------------------------------
//
// Title       : Chien_cell
// Design      : decoder_5bit
// Author      : Sindre
// File        : Chien_cell.v
//
//------------------------------------------------------------
//
// Description : Chien search cell
//
//------------------------------------------------------------
`timescale 1 ns / 1 ps

//{module {Chien_cell}}
module Chien_cell #(parameter width=5) (
	input clk,				//clock signal
	input reset,			//reset signal
	input enable,			//enable signal
	input sel,				//select signal
	input [width-1:0] L,	//error locator coefficient input
	input [width-1:0] a,	//generator root input value
	output [width-1:0] out	//chien cell output
	);
	
	//internal wires
	wire [width-1:0] NET01;
	wire [width-1:0] NET02;
	
	//module instantiations
	
	//multiplexor
	MUX #(.width(width)) MUX01(
	.in1	(L),	 //input 1
	.in2	(out),	 //input 2
	.sel	(sel),	 //select signal
	.out1	(NET01)	 //output
	);
	
	//full multiplier
	Full_mult #(.width(width)) FFM01(
	.A	(NET01),	//input 1
	.B	(a),		//input 2
	.Y	(NET02)		//output
	);
	
	//D-register
	D_reg #(.width(width)) REG01(
	.clk	(clk),	  //clock signal
	.reset	(reset),  //reset signal
	.enable	(enable), //enable signal
	.d_in	(NET02),  //input
	.d_out	(out)	  //output
	);

endmodule
