//-----------------------------------------------------------------------------
//
// Title       : Chien_cell_W
// Design      : decoder_5bit
// Author      : Sindre
// File        : Chien_cell_W.v
//
//-----------------------------------------------------------------------------
//
// Description : Chien cell for calculation of omega(x), same structure as
//				 orignial Chien cell
//
//-----------------------------------------------------------------------------
`timescale 1 ns / 1 ps

//{module {Chien_cell_W}}
module Chien_cell_W  #(parameter width=5) (
	input clk,			   //clock signal
	input reset,		   //reste signal
	input enable,		   //enable signal
	input sel,			   //select signal
	input [width-1:0] W,   //error evaluator coefficient input
	input [width-1:0] a,   //generator root input value
	output [width-1:0] out //output
	);
	
	//internal wires
	wire [width-1:0] NET01;
	wire [width-1:0] NET02;
	
	//module instantiations
	MUX #(.width(width)) MUX01(
	.in1	(W),	//input 1
	.in2	(out),	//input 2
	.sel	(sel),	//selecct signal
	.out1	(NET01)	//output
	);
	
	//full multiplier
	Full_mult #(.width(width)) FFM01(
	.A	(NET01),  //input 1
	.B	(a),	  //input 2
	.Y	(NET02)	  //output
	);
	
	//D-register
	D_reg #(.width(width)) REG01(
	.clk	(clk),	  //clock signal
	.reset	(reset),  //reset signal
	.enable	(enable), //enable signal
	.d_in	(NET02),  //input
	.d_out	(out)	  //output
	);
	
endmodule
