//-----------------------------------------------------------------------------
//
// Title       : Codeword_Store_modified
// Design      : Decoder_Design_2
// Author      : Sindre
// File        : Codeword_Store_modified.v
//
//-----------------------------------------------------------------------------
//
// Description : Modified codeword storage module. Enables parallel reading
//				 of data from memory if re1 is selected
//
//-----------------------------------------------------------------------------
`timescale 1 ns / 1 ps

//{module {Codeword_Store_modified}}
module Codeword_Store_modified #(parameter width=5)(
	input clk,						  	//clock signal
	input we,						  	//write enable
	input re,						  	//read enable
	input [width-1:0] address_write1,  	//write address
	input [width-1:0] address_write2,  	//write address
	input [width-1:0] address_read,  	//read address
	input [width-1:0] data_in1,		  	//data input 1
	input [width-1:0] data_in2,		  	//data input 2
	output reg [width-1:0] data_out, 	//data output
	);
	
	reg [width-1:0] mem [0:30];
	reg [4:0] i;
	
	initial
	begin
		for(i=0; i<31; i=i+1)
			mem[i] = 0; 
	end
	
	always @(posedge clk)
	begin
		if(we == 1'b1)begin
			mem[address_write1] <= data_in1;
			mem[address_write2] <= data_in2;
		end
		
		if(re == 1'b1)begin
			data_out <= mem[address_read];
		end
	end

endmodule
