//-----------------------------------------------------------------------------
//
// Title       : decoder_top_tb
// Design      : decoder_5bit
// Author      : Sindre
// File        : decoder_top_tb.v
//
//-----------------------------------------------------------------------------
//
// Description : Testbench for decoder with two parallel syndrome cells. 
//
//-----------------------------------------------------------------------------

//remeber to set input values from text file!!!

`timescale 1ns / 1ps
module decoder_top_2_2_tb;
//Parameters declaration: 
defparam UUT.width = 5;
parameter width = 5;

//Internal signals declarations:
reg clk = 1'b0;
reg reset;
reg enable = 1'b0;
reg rs_start;
reg [width-1:0]data_in1;
reg [width-1:0]data_in2;
wire NO_ERRORS;
wire NO_COR;
wire rs_done;
wire [width-1:0]data_out;


//parameter declerations
parameter words = 1; // number of test vectors to read

//internal TB declerations:
integer i, l, k, k1, k2, t;
integer error;
integer errors;
integer passed_tests;
reg NEW_TEST = 0;
reg START = 0;
reg CON = 0;
reg [width-1:0] symbol [0:30]; 		//memory for storing input test vector, current test vectors: 20
reg [width-1:0] mem_check [0:30];	//storage for decoded symbol values, used for verification

//file declaration
integer test_values;	//test vector file
integer check_values;	//golden values, calculated using Matlab
integer report_file; 	//LOG file
integer f1, f2;	
	
//wait time declaration
parameter clk_period = 12.5; // half clock period
parameter wait2clk = clk_period*2;

// Unit Under Test port map
	decoder_top UUT (
		.clk(clk),
		.reset(reset),
		.enable(enable),
		.rs_start(rs_start),
		.data_in1(data_in1),
		.data_in2(data_in2),
		.NO_ERRORS(NO_ERRORS),
		.NO_COR(NO_COR),
		.rs_done(rs_done),
		.data_out(data_out));

// clock generator
// 40Mhz clock
always begin
	#clk_period clk = 1'b1;
	#clk_period clk = 1'b0;
end

//Open and read from test vector file
initial begin
	test_values = $fopen("<input_file_name_here>.txt", "r");//change input test values before testing!!
	
	report_file = $fopen("output_stim.txt", "w");
	$display(" << Reading from text file >>");
	
	//read from test values
	while ( ! $feof(test_values))begin
		for(k1=0; k1<words; k1=k1+1)begin
			if(k1 == 0)begin
				for(l=0; l<31; l=l+1)begin
					f1 = $fscanf(test_values, "%d", symbol[l][4:0]);
				end //end for
			end else begin
				@(posedge NEW_TEST)
				for(l=0; l<31; l=l+1)begin
					f1 = $fscanf(test_values, "%d", symbol[l][4:0]);
				end //end for
			end //end if
		end	//end for
	end //end while loop
		
	$display(" << End of test vector file >>");
		
	$fclose(test_values);	//close test value file
		
end //end initial

initial begin
	check_values = $fopen("<input_file_name_here>.txt", "r");
	
	//read from check values (data decoded with matlab)
	while ( ! $feof(check_values))begin
		for(k2=0; k2<words; k2=k2+1)begin
			if(k2 == 0)begin
				for(l=0; l<31; l=l+1)begin
					f2 = $fscanf(check_values, "%d", mem_check[l][4:0]);
				end //end for
			end else begin
				@(posedge NEW_TEST)
				for(l=0; l<31; l=l+1)begin
					f2 = $fscanf(check_values, "%d", mem_check[l][4:0]);
				end //end for
			end//end if
		end	//end for
	end //end while loop
	
	$fclose(check_values);	//close check value file
	
end
	
initial begin
	$display(" << Starting simulation @ %0dns >>", $time);
	passed_tests = 0;
	error = 0;
	errors = 0;
	
	reset = 1'b1;
	rs_start = 1'b0;
	START = 0;
	
	#wait2clk reset = 1'b0; //reset RS decoder module
	#wait2clk reset = 1'b1;
	
	@(posedge clk)
	START = 1;
	//#wait2clk
	enable = 0;
	
	for(t=0; t<words; t=t+1)begin
		@(posedge CON)
		NEW_TEST = 1;
		START = 1;
		enable = 1'b0;
	end
end 


initial begin
	$fwrite(report_file, "Test report for RS decoder");
	
	//codeword input
	for(k=0; k<words; k=k+1)begin
		@(posedge START)
		
		//read in symbols by symbol
		for(i=0; i<31; i=i+2)begin
			if(i == 0) begin
				@(posedge clk)
				CON = 0;
				NEW_TEST = 0;
				START = 0;
				enable = 1'b1;
				rs_start = 1'b1; 		//start data is high one clk
				data_in1 = symbol[i]; 	//input first symbol
				data_in2 = symbol[i+1]; 	//input first symbol
			end else if(i == 30)begin
				@(posedge clk)
				START = 0;
				rs_start = 1'b0;
				data_in1 = symbol[i]; 	//input first symbol
				data_in2 = 0;
			end else begin 
				@(posedge clk)
				START = 0;
				rs_start = 1'b0;
				data_in1 = symbol[i];
				data_in2 = symbol[i+1];
			end
		end//end loop
		
		//check control signals while decoding
		@(posedge rs_done or posedge NO_ERRORS or posedge NO_COR)
		if(rs_done == 1)begin 
			$display("\n << Test vector %0d >> ", k+1);
			for(i=0; i<31; i=i+1)begin
				@(posedge clk)
				if(NO_ERRORS == 1 || NO_COR == 1)begin
					if(NO_ERRORS == 1)begin
						i = 31;
						$fwrite(report_file, "\n NO ERRORS on test vector %0d!", k+1);
						$display("\n << NO ERRORS on test vector %0d! >>", k+1);
					end else if(NO_COR == 1)begin
						i = 31;
						$fwrite(report_file, "\n Too many errors to correct on test vector %0d!", k+1);
						$display("\n << Too many errors to correct on test vector %0d! >>", k+1);
					end
				end else begin
					//check symbol output from decoder
					if(mem_check[i] == data_out)begin
						//if decoded values matches "golden values" decoding is correct
						$display(" %0d ", i+1);				
					end else begin
						//if no match, output error message
						$display(" << Error on symbol %0d >> ", i+1);
						error = error+1;
					end
				end
			end
		end
		//checks for different error flags
		if(!NO_ERRORS)begin //if no error display message in console
			if(!NO_COR)begin
				if(error)begin
					errors = errors+1;
					error = 0;
					$fwrite(report_file, "\nERROR on test vector %0d!", k+1);
				end else begin
					$fwrite(report_file, "\nTest vector %0d passed!", k+1);			
				end
			end
		end else if(NO_ERRORS == 1 || NO_COR == 1)begin
			if(NO_ERRORS == 1)begin
				$fwrite(report_file, "\nNO ERRORS on test vector %0d!", k+1);
				$display("\n << NO ERRORS on test vector %0d! >>", k+1);
			end else if(NO_COR == 1)begin
				$fwrite(report_file, "\nToo many errors to correct on test vector %0d!", k+1);
				$display("\n << Too many errors to correct on test vector %0d! >>", k+1);
			end
		end
	
		@(posedge clk)
		CON = 1;	//start new input sequence
	end//end input code word loop 	
	
	//output number of errors
	if(errors)begin
		//if errors, print message in console
		$display(" << Errors were encountered, check output.txt file >> ");
	end else begin
		//print total simulation time in console
		$display(" << Simulation ended after %0dns >> ", $time);
	end

	passed_tests = words-errors; //find number of passed tests
	
	//print number of passed tests
	$display(" << %0d of %0d test vectors passed >> ", passed_tests, words);
	$fwrite(report_file, "\n%0d of %0d test vectors passed", passed_tests, words);
		 
	$fclose(report_file); //close report file
	
	$finish; //end simulation
	
end

endmodule
