//---------------------------------------------------------------
//
// Title       : e_ctrl
// Design      : RS_encoder
// Author      : Sindre
// File        : e_ctrl.v
//
//---------------------------------------------------------------
//
// Description :	encoder control module
//
//---------------------------------------------------------------
`timescale 1 ns / 1 ps

//{module {e_ctrl}}
module e_ctrl (
	input clk,				//clock signal
	input reset,			//reset signal
	input enable,			//enable signal
	input s_ready,			//start encoding signal
	input [4:0] data_in,	//symbol input
	output reg ctrl,	   	//control signal for AND gate and MUX
	output reg finish,		//finish signal
	output reg [4:0] symbol	//symbol output
	);
	
	//constant declaration
	parameter size = 4;
	parameter IDLE = 4'b0000;
	parameter STEP1 = 4'b0001;
	parameter STEP2 = 4'b0011;

	//internal variables
	reg [size-1:0] state;
	reg [size-1:0] next_state;
	reg [5:0] i;				//internal counter
	
	//---------------FSM----------------//
	always @(posedge clk or negedge reset)
	begin: FSM_SEQ
		if(reset == 1'b0)begin
			state <= IDLE;
		end else if(enable) begin
			state <= next_state;
		end
	end
	
	 //Function for Combo Logic
	always @(state or s_ready or i)
	begin: FSM_COMBO
		case(state)
			IDLE: begin
				if(s_ready == 1)begin
					next_state = STEP1;
				end else begin
					next_state = IDLE;
				end
			end
				
			STEP1: begin 
				if(i == 4)begin
					next_state = STEP2;
				end else begin
					next_state = STEP1;
				end 
			end
				
			STEP2: begin
				if(i == 1)begin
					next_state = IDLE;
				end else begin
					next_state = STEP2;
				end 
			end

			default: next_state = IDLE;
		endcase
	end
	
	//output section
	always @(posedge clk or negedge reset)
	begin: FSM_OUT
		if(reset == 1'b0)begin
			i <= 0;
			finish <= 0;
			ctrl <= 0;
			symbol <= 0;
		end else if(enable) begin
			case(state)
				IDLE: begin //idle state
					if(s_ready == 1)begin //start encoding
						i <= 30;
						finish <= 0;
						ctrl <= 1;
						symbol <= data_in; //shift symbols in	
					end else begin
						i <= 0;
						finish <= 0;
						ctrl <= 0;
						symbol <= 0;
					end
				end
				
				STEP1: begin //shift symbols into encoder
					if(i == 6)begin
						i <= i-1;
						ctrl <= 1;
						symbol[4:2] <=data_in[4:2]; 
						symbol[1:0] <= 2'b00; //pad last symbols with zero
					end else if(i == 5)begin
						i <= i-1;
						ctrl <= 1;
						symbol <=0; //pad last symbols with zero
					end else if(i == 4)begin
						i <= i-1;
						ctrl <= 0;
						symbol <= 0;
					end else begin
						i <= i-1;
						ctrl <= 1;
						symbol <= data_in; //shift symbol in
					end
				end
				
				STEP2: begin //wait while parity symbols shifte out
					if(i == 1)begin
						i <= 0;
						ctrl <= 0;
						finish <= 1; //encoding finish
						symbol <= 0;
					end else begin
						i <= i-1;
						ctrl <= 0;
						finish <= 0;
						symbol <= 0;
					end 	
				end

				default: begin
					i <= 0;
					finish <= 0;
					ctrl <= 0;
					symbol <= 0;
				end
			endcase
		end
	end
endmodule