//----------------------------------------------------------------
//
// Title       : encoder_top
// Design      : RS_encoder
// Author      : Sindre
// File        : encoder_top.v
//
//----------------------------------------------------------------
//
// Description : Encoder top module, connecting all sub modules				 
//
//----------------------------------------------------------------
`timescale 1 ns / 1 ps

//{module {encoder_top}}
module encoder_top (
	clk,		//clock signal
	reset,		//reset signal
	enable,		//enable signal
	i_start,	//start encoding process
	in,			//input bus
	finish,		//encoding finsh signal
	out			//output bus
	);
	
	//input signals
	input clk;
	input reset;
	input enable;
	input i_start;
	input [4:0] in;
	//output signals
	output finish;
	output [4:0] out;
	//port data type
	wire clk;
	wire reset;
	wire enable;
	wire i_start;
	wire [4:0] in;
	wire finish;
	wire [4:0] out;
	
	//internal wires
	wire EN;
	wire GCLK;
	wire S_READY;
	wire CTRL;
	wire [4:0] DATA;
	wire [4:0] SYMBOL;
	wire [4:0] NET00; 
	wire [4:0] NET01;
	wire [4:0] NET02;
	wire [4:0] NET03;
	wire [4:0] NET04;
	wire [4:0] NET05;
	wire [4:0] NET06;
	wire [4:0] NET07;
	wire [4:0] NET08;
	wire [4:0] NET09;
	wire [4:0] NET10;
	wire [4:0] NET11;
	wire [4:0] NET12;
	
	assign EN = enable;
	
	//connecting modules
	Mult_x10_5bit FFM00(
	.mult_in	(NET00),
	.mult_out	(NET01)
	);
	
	Mult_x23_5bit FFM01(
	.mult_in	(NET00),
	.mult_out	(NET03)
	);
	
	Mult_x19_5bit FFM02(
	.mult_in	(NET00),
	.mult_out	(NET06)
	);

	Mult_x15_5bit FFM03(
	.mult_in	(NET00),
	.mult_out	(NET09)
	);
	
	D_reg REG00(
	.clk	(clk),
	.reset	(reset),
	.enable	(EN),
	.d_in	(NET01),
	.d_out	(NET02)
	);

	D_reg REG01(
	.clk	(clk),
	.reset	(reset),
	.enable	(EN),
	.d_in	(NET04),
	.d_out	(NET05)
	);

	D_reg REG02(
	.clk	(clk),
	.reset	(reset),
	.enable	(EN),
	.d_in	(NET07),
	.d_out	(NET08)
	);
	
	D_reg REG03(
	.clk	(clk),
	.reset	(reset),
	.enable	(EN),
	.d_in	(NET10),
	.d_out	(NET11)
	);
	
	Adder FFA00(
	.in1	(NET02),
	.in2	(NET03),
	.sum	(NET04)
	);
	
	Adder FFA01(
	.in1	(NET05),
	.in2	(NET06),
	.sum	(NET07)
	);
	
	Adder FFA02(
	.in1	(NET08),
	.in2	(NET09),
	.sum	(NET10)
	);
	
	Adder FFA03(
	.in1	(NET11),
	.in2	(SYMBOL),
	.sum	(NET12)
	);
	
	MUX MUX00(
	.in1	(NET11),
	.in2	(SYMBOL),
	.sel	(CTRL),
	.out1	(out)
	);
	
	AND_gate AND00(
	.a		(NET12),
	.b		(CTRL),
	.y		(NET00)
	);
	
	e_ctrl CONTROL(
	.clk		(clk),
	.reset		(reset),
	.enable		(EN),
	.s_ready	(i_start),
	.data_in	(in),
	.ctrl		(CTRL),
	.finish		(finish),
	.symbol		(SYMBOL)
	);

endmodule
