//-----------------------------------------------------------------------------
//
// Title       : encoder_top_tb
// Design      : RS_encoder
// Author      : Sindre
// File        : encoder_top_TB.v
//
//-----------------------------------------------------------------------------
//
// Description :  RS encoder testbech
//
//-----------------------------------------------------------------------------

`timescale 1ns / 1ps
module encoder_top_tb;

//Parameters declaration: 
//defparam UUT.width = 5;
parameter width = 5;	// symbol width - 5 bit
parameter symbols = 27; // information symbols
parameter words = 20;	// number of test vectors/data packets to be encoded
parameter parity = 4;	// number of parity symbols
parameter size = 130;	// size of input data packet (bit)

//Internal signals declarations:
reg clk = 1'b0;
reg reset;
reg enable = 0;
reg i_start;
reg [4:0] in;
wire finish;
wire [width-1:0]out;

//internal values
integer i, l, i2, l2, index, k, s, index1; //counters
integer passed_tests; 	//counter to check number of passed tests
integer error_counter;	//error counter
reg OUT_FLAG;
//file declaration
integer test_values;	//test vector file
integer check_values;	//golden values, calculated using Matlab
integer report_file; 	//LOG file
integer f1, f11, f2;

//memory declaration for reading from file
reg [size-1:0] mem[0:words-1];
reg [width-1:0] mem1 [0:words-1];
reg [(width*(symbols+parity))-1:0] mem_check[0:words-1];

//memory declaration for storing encoded symbols;
reg [(width*(symbols+parity))-1:0] outmem[0:words-1];

//wait time declaration
parameter clk_period = 12.5; // clock period
parameter wait2clk = clk_period*2;

// Unit Under Test port map
	encoder_top UUT (
		.clk(clk),
		.reset(reset),
		.enable(enable),
		.i_start(i_start),
		.in(in),
		.finish(finish),
		.out(out));

// clock generator
// 40Mhz clock
always begin
	#clk_period clk = 1'b1;
	#clk_period clk = 1'b0;
end	
		
//Open and read from test vector file
initial
	begin
		test_values = $fopen("stimulus.txt", "r");
		check_values = $fopen("stimulus_encoded.txt", "r");
		report_file = $fopen("output_stim.txt", "w");
		$display(" << Reading from text file >>");
		
		//read from test values
		while ( ! $feof(test_values)) 
		begin
			for(l=0; l<words; l=l+1)
			begin
				for(i=129; i>3; i=i-5)
					begin
						f1 = $fscanf(test_values, "%d", mem[l][i -: 5]);
					end //end for
			end	//end for
		end //end while loop
		
		//read from check values (data encoded with matlab)
		while ( ! $feof(check_values)) 
		begin
			for(l2=0; l2<words; l2=l2+1)
			begin
				for(i2=154; i2>3; i2=i2-5)
				 	begin
					f2 = $fscanf(check_values, "%d", mem_check[l2][i2 -: 5]);
					end //end for
			end	//end for
		end //end while loop
		
		$display(" << End of test vector file >>");
		
		$fclose(test_values);	//close test value file
		$fclose(check_values);	//close check value file	
	end //end initial
	
initial
begin
	$display(" << Starting simulation @ %0dns >>", $time);
	reset = 1'b1;
	i_start = 0;
	#wait2clk reset = 1'b0; // reset is active (active low reset)
	#wait2clk reset = 1'b1;	//release reset after 25 ns
	OUT_FLAG = 0;		   	//reset OUTPUT FLAG
	error_counter = 0; 		//reset error counter

	for(index=0; index<words; index=index+1)
	begin
		@(posedge clk)
		enable = 1;
		@(posedge clk)
		i_start = 1'b1;
		in = mem[index][129 -: 5];
		OUT_FLAG = 1; //flag used for reading output of encoder
		for(k=124; k>3; k=k-5)begin
			@(posedge clk)
			i_start = 1'b0;
			in = mem[index][k -: 5];
		end
		
		//wait for finish signal
		@(negedge finish)
		OUT_FLAG = 0;
		enable = 0;
		
		//switching activity wait time
		//#2400;
		
		// check output from encoder
		if(outmem[index] !== mem_check[index])
		begin
			error_counter = error_counter+1; //if mismatch, increment error counter by one
			//write the encoded data to file if there is no match between encoded and expected values
			$display(" << [%0dns] ERROR! The encoded value does not match expected value >> ", $time);
			$fwrite(report_file, "ERROR on test vector %0d! The encoded value does not match expected value\n", index+1);
			for(i=154; i>3; i=i-5)
				$fwrite(report_file, "%0d ", outmem[index][i -: 5]);
				$fwrite(report_file, "\n");
		end else 
		begin
			$fwrite(report_file, "MATCH on test vector %0d! The encoded value matched the expected value\n", index+1);
		end //end if else	
	end//end for loop
	
	//if errors, print message in console
	if(error_counter)
		$display(" << Errors were encountered, check output.txt file >> ");
	else
		//print total simulation time in console
		$display(" << Simulation ended after %0dns >> ", $time);
		 
	passed_tests = words-error_counter; //find number of passed tests
		 
	//print number of passed tests
	$display(" << %0d of %0d test vectors passed >> ", passed_tests, words);
	$fwrite(report_file, "%0d of %0d test vectors passed", passed_tests, words);
		 
	$fclose(report_file); //close report file
	$display(" << Simulation ended after %0dns >> ", $time);
	$finish; //end simulation
	
end

//intitial used to read from output of encoder and into memout
initial
begin
	
	for(index1=0; index1<words; index1=index1+1)begin
		@(posedge OUT_FLAG)
			for(s=154; s>3; s=s-5)begin
	   			@(posedge clk)
				outmem[index1][s -: 5] = out;
			end
	end 			
end 

endmodule
